<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
                xmlns:wsse="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"
                xmlns:medcom="http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd"
                xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/">
    <xsl:output method="xml" indent="yes"/>

    <!--  Tilføjer nogle flere schema'er efter den først fundne -->
    <xsl:template match="xs:schema">
        <xsl:copy-of select="."/>

        <xs:schema targetNamespace="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
            <xs:include schemaLocation="../../xsd/dgws/wsse.xsd" />
        </xs:schema>
        <xs:schema targetNamespace="urn:oasis:names:tc:SAML:2.0:assertion">
            <xs:include schemaLocation="../../xsd/dgws/saml.xsd" />
        </xs:schema>
        <xs:schema targetNamespace="http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd">
            <xs:include schemaLocation="../../xsd/dgws/medcom.xsd" />
        </xs:schema>
    </xsl:template>

    <!--  Tilføjer flere messages lige efter "wsdl:types" -->
    <xsl:template match="wsdl:types">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>

        <wsdl:message name="SecurityHeader"><wsdl:part name="security_header" element="wsse:Security"/></wsdl:message>
        <wsdl:message name="MedcomHeader"><wsdl:part name="medcom_header" element="medcom:Header"/></wsdl:message>
        <wsdl:message name="DGWSFault"><wsdl:part name="DGWSFault" element="medcom:FaultCode"/></wsdl:message>
    </xsl:template>

    <!-- Under Binding/Operation laves en ny udgave af <wsdl:input> med nye security headers. -->
    <xsl:template match="wsdl:binding/wsdl:operation/wsdl:input">
        <wsdl:input>
            <soap:header message="tns:SecurityHeader" part="security_header" use="literal" />
            <soap:header message="tns:MedcomHeader" part="medcom_header" use="literal" />
            <soap:body use="literal"/>
        </wsdl:input>
    </xsl:template>

    <!--
        Det samme som forrige, bare for <wsdl:output>.
        Derudover tilføjes DGWSFault lige under <wsdl:output>.
    -->
    <xsl:template match="wsdl:binding/wsdl:operation/wsdl:output">
        <wsdl:output>
            <soap:header message="tns:SecurityHeader" part="security_header" use="literal" />
            <soap:header message="tns:MedcomHeader" part="medcom_header" use="literal" />
            <soap:body use="literal"/>
        </wsdl:output>
        <wsdl:fault name="DGWSFault">
            <soap:fault name="DGWSFault" use="literal"/>
        </wsdl:fault>
    </xsl:template>

    <!-- DGWSFault skal også tilføjes lige efter output i operations under portType -->
    <xsl:template match="wsdl:portType/wsdl:operation/wsdl:output">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>
        <wsdl:fault name="DGWSFault" message="tns:DGWSFault"/>
    </xsl:template>

    <!-- Alle andre attributter og noder kopieres 1-1 -->
    <xsl:template match="@*|node()">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>
    </xsl:template>

    <!-- Vi har brug for at de nye namespaces sættes på rodelementet, dvs på <wsdl:definitions> -->
    <xsl:template match="wsdl:definitions">
        <wsdl:definitions xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
                          xmlns:xs="http://www.w3.org/2001/XMLSchema"
                          xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/"
                          xmlns:tns="http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/"
                          xmlns:medcom="http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd"
                          xmlns:wsse="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"
                          targetNamespace="http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/">
            <xsl:apply-templates select="@*|node()"/>
        </wsdl:definitions>
    </xsl:template>
</xsl:stylesheet>