package dk.nsp.idsas.dao;

import dk.nsp.idsas.util.ErrorMessages;
import org.apache.log4j.Logger;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import javax.sql.DataSource;
import java.util.HashMap;

public class SaltDao {
    private static final Logger logger = Logger.getLogger(SaltDao.class);

    private final NamedParameterJdbcTemplate jdbcTemplate;

    public SaltDao(DataSource dataSource) {
        jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
    }

    public byte[] getLatestSalt() {
        String query = "SELECT salt FROM salt ORDER BY created_time DESC, id DESC LIMIT 1";
        try {
            return jdbcTemplate.queryForObject(query, new HashMap<>(), byte[].class);
        } catch (EmptyResultDataAccessException e) {
            logger.error(e.getMessage(), e);
            throw new RuntimeException(ErrorMessages.noSaltError());
        }
    }

    public void insert(byte[] salt) {
        String query = "INSERT INTO salt(salt) VALUES (:salt)";
        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("salt", salt);

        jdbcTemplate.update(query, parameters);
    }
}
