package dk.nsp.idsas.dao;

import dk.nsp.idsas.util.WhitelistType;
import org.apache.log4j.Logger;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import javax.sql.DataSource;
import java.util.HashMap;

public class WhitelistDao {
    private static final Logger logger = Logger.getLogger(WhitelistDao.class);

    private final NamedParameterJdbcTemplate jdbcTemplate;

    public WhitelistDao(DataSource dataSource) {
        jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
    }

    public boolean isWhitelisted(String actorId, WhitelistType type) {
        String query = "select 1 from whitelist where actor_id = :actor_id and type = :type";
        HashMap<String, String> parameters = new HashMap<>();
        parameters.put("actor_id", actorId);
        parameters.put("type", type.name());
        try {
            jdbcTemplate.queryForObject(query, parameters, Boolean.class);
            return true;
        } catch (EmptyResultDataAccessException e) {
            logger.error(e.getMessage(), e);
            return false;
        }
    }
}
