package dk.nsp.idsas.security;

import dk.nsp.idsas.security.impl.IdsasActorEnricherImpl;
import dk.nsp.idsas.security.impl.IdsasActorProviderImpl;
import dk.nsp.idsas.security.impl.IdsasActorTransformerImpl;
import dk.nsp.idsas.security.impl.IdsasActorValidatorImpl;
import dk.sds.nsp.security.SecurityContext;

import java.util.function.Supplier;
import org.apache.log4j.Logger;

public class IdsasActorBuilder {
    private static final Logger logger = Logger.getLogger(IdsasActorBuilder.class);

    private final Supplier<SecurityContext> securitySupplier;

    public IdsasActorBuilder(Supplier<SecurityContext> securitySupplier) {
        this.securitySupplier = securitySupplier;
    }

    /**
     * Bygger en generel actor til registration, lookup og getSalt.
     */
    public IdsasActor buildActor() throws IdsasSecurityException {
        IdsasActorProvider provider = new IdsasActorProviderImpl(securitySupplier);
        IdsasActorTransformer transformer = new IdsasActorTransformerImpl();
        IdsasActorEnricher enricher = new IdsasActorEnricherImpl();
        IdsasActorValidator validator = new IdsasActorValidatorImpl(securitySupplier);

        IdsasActor actor = provider.getActor();
        actor = transformer.getActor(actor);
        actor = enricher.getActor(actor);
        actor = validator.getActor(actor);

        return actor;
    }
}
