package dk.nsp.idsas.ws;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

public abstract class AbstractDksServlet extends AbstractServlet {
	private static final String DATETIME_FORMAT = "yyyy-MM-dd'T'HH:mm:ss'Z'";
	private static final Logger logger = LogManager.getLogger(AbstractDksServlet.class);

	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
		logger.trace("dksconfig request");
		response.setContentType("application/xml");
		response.setStatus(200);
		response.addHeader("Cache-Control", "no-cache");
		String resp = dksConfig();
		response.getWriter().print(resp);
	}

	protected abstract String getResourceName();

	protected abstract String getDccEndpoint();

	protected java.net.URL getResource() {
		return getClass().getResource(getResourceName());
	}

	protected InputStream getResourceAsStream() {
		return getClass().getResourceAsStream(getResourceName());
	}


	private String dksConfig() throws IOException {
		URL url = getResource();
		ZonedDateTime zdt = ZonedDateTime.ofInstant(Instant.ofEpochMilli(url.openConnection().getLastModified()), ZoneId.systemDefault());
		DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(DATETIME_FORMAT);
		String timestamp = dateTimeFormatter.format(zdt);

		try (InputStream in = getResourceAsStream();
			 BufferedReader reader = new BufferedReader(new InputStreamReader(in))) {
			String out = reader.lines().collect(Collectors.joining());
			return getStringWithResolvedVariables(new StringBuilder(out), timestamp);
		}
	}

	private String getStringWithResolvedVariables(StringBuilder template, String timestamp) {
		int timestampIndex = template.indexOf("TIMESTAMP");
		template.replace(timestampIndex, timestampIndex + 9, timestamp);
		return template.toString().replace("ENDPOINT_PLACEHOLDER", getDccEndpoint());
	}
}
