package dk.nsp.idsas.util;

import dk.sds.nsp.audit.AuditBuilder;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.time.Instant;
import java.util.Base64;

public class AuditLoggerTest {

    private AuditBuilder auditBuilder;
    private AuditLogger auditLogger;

    @Before
    public void setUp() {
        auditBuilder = Mockito.mock(AuditBuilder.class);
        auditLogger = new AuditLogger(() -> auditBuilder);
    }

    @Test
    public void logCreateBlurring_logsCprAndCvr() {
        String cpr = "1111111111";
        String hashedCpr = "ABCDEF";
        String cvr = "1111";
        Instant endDateTime = Instant.now();
        auditLogger.logCreateBlurring(cpr, "cpr", hashedCpr, cvr, "cvr", endDateTime);

        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "createBlurring", AuditBuilder.TypeOfInformation.RegularPersonalInformation, "patient-id", cpr);
        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "createBlurring", AuditBuilder.TypeOfInformation.RegularPersonalInformation, "patient-id-source", "cpr");
        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "createBlurring", AuditBuilder.TypeOfInformation.NonPersonalInformation, "actor-id", cvr);
        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "createBlurring", AuditBuilder.TypeOfInformation.NonPersonalInformation, "actor-id-source", "cvr");
        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "createBlurring", AuditBuilder.TypeOfInformation.SensitivePersonalInformation, "hashed-patient-id", hashedCpr);
        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "createBlurring", AuditBuilder.TypeOfInformation.NonPersonalInformation, "expiry-date-time", endDateTime);
        Mockito.verifyNoMoreInteractions(auditBuilder);
    }

    @Test
    public void logGetCurrentSalt_logsCvr() {
        String cvr = "3333";
        String base64encodedSalt = Base64.getEncoder().encodeToString(new byte[]{1, 2, 3});
        auditLogger.logGetCurrentSalt(cvr, "cvr", new byte[]{1, 2, 3});

        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "getCurrentSalt", AuditBuilder.TypeOfInformation.NonPersonalInformation, "actor-id", cvr);
        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "getCurrentSalt", AuditBuilder.TypeOfInformation.NonPersonalInformation, "actor-id-source", "cvr");
        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "getCurrentSalt", AuditBuilder.TypeOfInformation.NonPersonalInformation, "base64-encoded-salt", base64encodedSalt);
        Mockito.verifyNoMoreInteractions(auditBuilder);
    }

    @Test
    public void logGetBlurredOrganisations_logsCpr() {
        String cpr = "2222222222";
        auditLogger.logGetBlurredOrganisations(cpr, "cpr");

        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "getBlurredOrganisations", AuditBuilder.TypeOfInformation.RegularPersonalInformation, "patient-id", cpr);
        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "getBlurredOrganisations", AuditBuilder.TypeOfInformation.RegularPersonalInformation, "patient-id-source", "cpr");
        Mockito.verifyNoMoreInteractions(auditBuilder);
    }
}