package dk.nsp.idsas.util;

import java.io.IOException;
import java.util.ArrayList;

import com.fasterxml.jackson.databind.ObjectMapper;
import dk.nsp.idsas.util.health.HealthIndication;
import dk.nsp.idsas.util.health.HealthIndicator;
import org.junit.Test;

import static org.junit.Assert.*;

public class HealthReportGeneratorTest {

    @Test
    public void reportResultIsJSON() {
        // Arrange and act
        String result = HealthReportGenerator.generateResult(new ArrayList<HealthIndicator>() {{
            add(() -> new HealthIndication(200, new JsonPair("Version", "1.0.0")));
            add(() -> new HealthIndication(200, new JsonPair("Test", "123")));
        }});

        // Assert
        try {
            final ObjectMapper objectMapper = new ObjectMapper();
            objectMapper.readTree(result);
        } catch (IOException e) {
            fail("Should not have failed on creating JSON parser");
        }
    }

    @Test
    public void reportStatus200() {
        // Arrange and act
        int result = HealthReportGenerator.generateStatus(new ArrayList<HealthIndicator>() {{
            add(() -> new HealthIndication(200, new JsonPair("Version", "1.0.0")));
            add(() -> new HealthIndication(200, new JsonPair("Test", "123")));
        }});

        // Assert
        assertEquals(200, result);
    }

    @Test
    public void reportStatus404() {
        // Arrange and act
        int result = HealthReportGenerator.generateStatus(new ArrayList<HealthIndicator>() {{
            add(() -> new HealthIndication(200, new JsonPair("Version", "1.0.0")));
            add(() -> new HealthIndication(404, new JsonPair("Test", "123")));
        }});

        // Assert
        assertEquals(404, result);
    }

}