package dk.nsp.idsas.ws;

import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.List;

public class AbstractWsdlServletTest {

    private AbstractWsdlServlet wsdlServlet;
    private HttpServletResponse httpResponse;
    private ServletOutputStream responseOutputStream;
    private HttpServletRequest httpRequest;

    @Before
    public void setUp() throws Exception {
        httpRequest = Mockito.mock(HttpServletRequest.class);
        httpResponse = Mockito.mock(HttpServletResponse.class);
        responseOutputStream = Mockito.mock(ServletOutputStream.class);
        Mockito.when(httpResponse.getOutputStream()).thenReturn(responseOutputStream);
        wsdlServlet = new AbstractWsdlServlet() {
            @Override
            List<String> validSuffixes() {
                return Arrays.asList(".test", ".wsdl");
            }
        };
    }

    @Test
    public void doGet_WhenPathDoesNotHaveValidSuffix_Returns404() throws IOException {
        Mockito.when(httpRequest.getPathInfo()).thenReturn("wsdl/test.xml");

        wsdlServlet.doGet(httpRequest, httpResponse);

        Mockito.verify(httpResponse).setStatus(404);
    }

    @Test
    public void doGet_WhenPathHaveValidSuffixButResourceDoesNotExist_Returns404() throws IOException {
        Mockito.when(httpRequest.getPathInfo()).thenReturn("wsdl/nonexisting.wsdl");

        wsdlServlet.doGet(httpRequest, httpResponse);

        Mockito.verify(httpResponse).setStatus(404);
    }

    @Test
    public void doGet_WhenRequestingExistingResourceWithValidSuffixWsdl_ReturnsWsdlAndStatus200() throws IOException {
        String path = "wsdl/test.wsdl";
        Mockito.when(httpRequest.getPathInfo()).thenReturn(path);
        InputStream is = this.getClass().getResourceAsStream("/" + path);
        byte[] wsdlContent = new byte[is.available()];
        is.read(wsdlContent);

        wsdlServlet.doGet(httpRequest, httpResponse);

        Mockito.verify(httpResponse).setStatus(200);
        Mockito.verify(responseOutputStream).write(wsdlContent);
    }

    @Test
    public void doGet_WhenRequestingExistingResourceWithValidSuffixTest_ReturnsTestFileAndStatus200() throws IOException {
        String path = "wsdl/test.test";
        Mockito.when(httpRequest.getPathInfo()).thenReturn(path);
        InputStream is = this.getClass().getResourceAsStream("/" + path);
        byte[] wsdlContent = new byte[is.available()];
        is.read(wsdlContent);

        wsdlServlet.doGet(httpRequest, httpResponse);

        Mockito.verify(httpResponse).setStatus(200);
        Mockito.verify(responseOutputStream).write(wsdlContent);
    }
}