package dk.nsp.idsas.test._20230601;

import dk.nsp.idsas.test.util.HttpClient;
import io.cucumber.java.en.Then;
import io.cucumber.java.en.When;
import org.springframework.beans.factory.annotation.Value;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class WsdlStepDefinitions {

    @Value("${idsas.registration.endpoint}" + "/20230601")
    private String registrationEndpoint;

    @Value("${idsas.registration.endpoint}" + "/service-contract/secure-wsdl/20230601")
    private String registrationSecureWsdlEndpoint;

    @Value("${idsas.lookup.endpoint}" + "/20230601")
    private String lookupServiceEndpoint;

    @Value("${idsas.lookup.endpoint}" + "/service-contract/secure-wsdl/20230601")
    private String lookupServiceSecureWsdlEndpoint;

    @Value("${idsas.salt.endpoint}" + "/20230601")
    private String saltEndpoint;

    @Value("${idsas.salt.endpoint}" + "/service-contract/secure-wsdl/20230601")
    private String saltSecureWsdlEndpoint;

    private Map<String, String> endpoints = new HashMap<>();
    private Map<String, String> secureEndpoints = new HashMap<>();

    private int responseCode;
    private String responseBody;

    @PostConstruct
    public void init() {
        endpoints.put("idsas-registration", registrationEndpoint);
        endpoints.put("idsas-lookup", lookupServiceEndpoint);
        endpoints.put("idsas-salt", saltEndpoint);
        secureEndpoints.put("idsas-registration", registrationSecureWsdlEndpoint);
        secureEndpoints.put("idsas-lookup", lookupServiceSecureWsdlEndpoint);
        secureEndpoints.put("idsas-salt", saltSecureWsdlEndpoint);
    }

    @When("{string} wsdl hentes fra {string}")
    public void getWsdl(String wsdl, String service) throws IOException {
        String endpoint = endpoints.get(service);
        String url = String.format("%s/%s?wsdl", endpoint, wsdl);

        HttpClient httpClient = new HttpClient();
        responseBody = httpClient.get(url);
        responseCode = httpClient.getResponseCode();
    }

    @When("{string} wsdl med security hentes fra {string}")
    public void getSecureWsdl(String wsdl, String service) throws IOException {
        String endpoint = secureEndpoints.get(service);
        String url = String.format("%s/idsas-%s.wsdl", endpoint, wsdl);

        HttpClient httpClient = new HttpClient();
        responseBody = httpClient.get(url);
        responseCode = httpClient.getResponseCode();
    }

    @Then("Returneres http kode {int}")
    public void validateResponseCode(int expectedResponseCode) {
        assertEquals(expectedResponseCode, responseCode);
    }

    @Then("Svaret indeholder wsdl definition")
    public void validateWsdlDefinitionInResponseBody() {
        assertTrue(responseBody.contains("<wsdl:definitions"));
    }

    @Then("Svaret indeholder security header")
    public void validateSecurityHeaderInResponseBody() {
        assertTrue(responseBody.contains("<wsdl:message name=\"SecurityHeader\""));
    }
}
