package dk.nsp.idsas.test._20240124;

import dk.nsp.idsas.test.idptest.SOAPOperation;
import dk.nsp.idsas.test.idptest.ServiceCaller;
import dk.nsp.idsas.test.setup.TestSetup;
import dk.nsp.idsas.test.util.TestIdentities;
import dk.nsp.test.idp.model.Identity;
import dk.sundhedsdatastyrelsen.identitetssloering._2024._01._24.*;
import io.cucumber.java.en.And;
import io.cucumber.java.en.Then;
import io.cucumber.java.en.When;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.net.URI;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class OrgBlurringStepDefinitions {

    @Value("${idsas.registration.endpoint}" + "/20240124/registration")
    private String idsasRegistrationUrl2024;

    @Value("${idsas.lookup.endpoint}" + "/20240124/lookup")
    private String idsasLookupUrl2024;

    @Autowired
    private TestSetup testSetup;

    private String response = "";

    @When("en {string} opretter en afdelingssløring for {string} {string}")
    public void enOpretterEnAfdelingssløringForCVR(String brugertype, String orgType, String orgId) {
        createOrgBlurring(brugertype, orgType, orgId);
    }

    @Then("giver den svaret {string}")
    public void såGiverDenSvaret(String svar) {
        assertEquals(svar, response);
    }

    @Then("sender den fejlen {string}")
    public void senderDenFejlen(String fejl) {
        assertTrue(response.contains(fejl));
    }

    @When("en {string} sletter en afdelingssløring for {string} {string}")
    public void enSletterEnAfdelingssløringForSOR(String bruger, String orgType, String orgId) {
        removeOrgBlurring(bruger, orgType, orgId);
    }

    @And("en systembruger lister alle aktive afdelingssløringer")
    public void enListerAfdelingssløringer() {
        listAllActiveOrgBlurrings("DRG");
    }

    @And("en systembruger med forkert clientName lister alle aktive afdelingssløringer")
    public void enSystembrugerMedForkertClientNameListerAlleAktiveAfdelingssløringer() {
        listAllActiveOrgBlurrings("ugyldigt-client-name");
    }

    @And("en {string} lister alle afdelingssløringer for et CVR")
    public void enListerAlleAfdelingssløringerForCVREtCVR(String bruger) {
        listOrgBlurringsForCVR(bruger);
    }

    @Then("sender den en svar der indeholder {string} og {string}")
    public void senderDenEnSvarDerIndeholder(String svar1, String svar2) {
        assertTrue(response.contains(svar1));
        assertTrue(response.contains(svar2));
    }

    private void createOrgBlurring(String userType, String orgType, String orgId) {
        CreateOrgBlurringRequest request = new CreateOrgBlurringRequest();
        OrganisationIdType blurredOrgId = new OrganisationIdType();
        blurredOrgId.setId(orgId);
        blurredOrgId.setClassification(OrganisationIdClassificationType.fromValue(orgType));
        request.setOrg(blurredOrgId);
        Identity identity = TestIdentities.identityFor(userType);

        try {
            ServiceCaller<CreateOrgBlurringRequest, String> serviceCaller =
                    new ServiceCaller<>(CreateOrgBlurringRequest.class, String.class);
            response = serviceCaller.call(identity,
                    request, SOAPOperation.CreateOrgBlurring.getSoapOperation(), new URI(idsasRegistrationUrl2024));
        } catch (Exception e) {
            response = e.getMessage();
        }
        System.out.println(response);
    }

    private void removeOrgBlurring(String userType, String orgType, String orgId) {
        RemoveOrgBlurringRequest removeOrgBlurringRequest = new RemoveOrgBlurringRequest();
        OrganisationIdType org = new OrganisationIdType();
        org.setId(orgId);
        org.setClassification(OrganisationIdClassificationType.fromValue(orgType));
        removeOrgBlurringRequest.setOrg(org);
        Identity identity = TestIdentities.identityFor(userType);

        try {
            ServiceCaller<RemoveOrgBlurringRequest, String> serviceCaller = new ServiceCaller<>(RemoveOrgBlurringRequest.class, String.class);
            response = serviceCaller.call(identity,
                    removeOrgBlurringRequest, SOAPOperation.RemoveOrgBlurring.getSoapOperation(), new URI(idsasRegistrationUrl2024));
        } catch (Exception e) {
            response = e.getMessage();
        }
    }

    private void listAllActiveOrgBlurrings(String clientName) {
        Identity identity = TestIdentities.systembrugerMedClientName(clientName);

        try {
            ServiceCaller<ListAllActiveOrgBlurringsRequest, ListAllActiveOrgBlurringsResponse> serviceCaller =
                    new ServiceCaller<>(ListAllActiveOrgBlurringsRequest.class, ListAllActiveOrgBlurringsResponse.class);
            ListAllActiveOrgBlurringsResponse res = serviceCaller.call(identity,
                    new ListAllActiveOrgBlurringsRequest(), SOAPOperation.ListAllActiveOrgBlurrings.getSoapOperation(), new URI(idsasLookupUrl2024));
            response = res.getOrg().stream().map(e -> e.getId() + e.getClassification()).reduce("", (a, b) -> a + b);
        } catch (Exception e) {
            response = e.getMessage();
        }
    }

    private void listOrgBlurringsForCVR(String userType) {
        Identity identity = TestIdentities.identityFor(userType);

        try {
            ServiceCaller<ListOrgBlurringsForCVRRequest, ListOrgBlurringsForCVRResponse> serviceCaller =
                    new ServiceCaller<>(ListOrgBlurringsForCVRRequest.class, ListOrgBlurringsForCVRResponse.class);
            ListOrgBlurringsForCVRResponse res = serviceCaller.call(identity,
                    new ListOrgBlurringsForCVRRequest(), SOAPOperation.ListOrgBlurringsForCVR.getSoapOperation(), new URI(idsasLookupUrl2024));
            response = res.getOrg().stream().map(e -> e.getId() + e.getClassification()).reduce("", (a, b) -> a + b);
        } catch (Exception e) {
            response = e.getMessage();
        }
    }
}