package dk.nsp.idsas.integrations;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.concurrent.ConcurrentLinkedQueue;

public class ServiceErrorCollector {
    private final Duration errorCountDuration;
    private final ConcurrentLinkedQueue<LocalDateTime> errorTimestamps = new ConcurrentLinkedQueue<>();

    public ServiceErrorCollector(Duration errorCountDuration) {
        this.errorCountDuration = errorCountDuration;
    }

    public void registerServiceError() {
        errorTimestamps.add(LocalDateTime.now());
    }

    public void registerServiceSuccess() {
        errorTimestamps.clear();
    }

    public int getErrorCount() {
        LocalDateTime from = LocalDateTime.now().minus(errorCountDuration);
        errorTimestamps.removeIf(timestamp -> timestamp.isBefore(from));
        return errorTimestamps.size();
    }
}
