package dk.nsp.idsas.integrations;

import org.junit.Test;

import java.time.Duration;

import static org.junit.Assert.assertEquals;

public class ServiceErrorCollectorTest {

    @Test
    public void getErrorCount_WhenNothingHasBeenRegistered_Returns0() {
        ServiceErrorCollector serviceErrorCollector = new ServiceErrorCollector(Duration.ofMinutes(10));

        int errorCount = serviceErrorCollector.getErrorCount();

        assertEquals(0, errorCount);
    }

    @Test
    public void getErrorCount_WhenSuccessHasBeenRegistered_Returns0() {
        ServiceErrorCollector serviceErrorCollector = new ServiceErrorCollector(Duration.ofMinutes(10));
        serviceErrorCollector.registerServiceSuccess();

        int errorCount = serviceErrorCollector.getErrorCount();

        assertEquals(0, errorCount);
    }

    @Test
    public void getErrorCount_WhenTwoErrorsHasBeenRegisteredWithinTimeLimit_Returns2() {
        ServiceErrorCollector serviceErrorCollector = new ServiceErrorCollector(Duration.ofMinutes(10));
        serviceErrorCollector.registerServiceError();
        serviceErrorCollector.registerServiceError();

        int errorCount = serviceErrorCollector.getErrorCount();

        assertEquals(2, errorCount);
    }

    @Test
    public void getErrorCount_WhenTwoErrorsHasBeenRegisteredBeforeTimeLimit_Returns0() {
        ServiceErrorCollector serviceErrorCollector = new ServiceErrorCollector(Duration.ofMinutes(-1));
        serviceErrorCollector.registerServiceError();
        serviceErrorCollector.registerServiceError();

        int errorCount = serviceErrorCollector.getErrorCount();

        assertEquals(0, errorCount);
    }

    @Test
    public void getErrorCount_WhenTwoErrorsHasBeenRegisteredWithinTimeLimitButSuccessAfterwards_Returns0() {
        ServiceErrorCollector serviceErrorCollector = new ServiceErrorCollector(Duration.ofMinutes(10));
        serviceErrorCollector.registerServiceError();
        serviceErrorCollector.registerServiceError();
        serviceErrorCollector.registerServiceSuccess();

        int errorCount = serviceErrorCollector.getErrorCount();

        assertEquals(0, errorCount);
    }
}