package dk.nsp.idsas.setup;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.OrgBlurringDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.http.PoolingHttpClientFactory;
import dk.nsp.idsas.integrations.ServiceErrorCollector;
import dk.nsp.idsas.integrations.sores.SoresClient;
import dk.nsp.idsas.service.LookupService;
import dk.nsp.idsas.service.OrgBlurringService;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.health.SoresClientHealthIndicator;
import org.apache.http.client.HttpClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;

import java.time.Duration;

@Configuration
@PropertySources({
        @PropertySource("classpath:/META-INF/maven/dk.nsp.idsas/idsas-lookup-web/pom.properties"),
        @PropertySource(value = "idsas-lookup.properties", encoding = "ISO-8859-1"),
        @PropertySource(value = "idsas-patient-id-salt.properties", encoding = "ISO-8859-1")
        // Bemærkning: I Java 8 er properties altid i ISO-8859-1, men fremadrettet skulle de gerne kunne være UTF-8.
})
public class WebSetup {

    @Value("${idsas.allowed.client.system.names}")
    private String[] allowedClientSystemNames;

    @Value("${idsas.patient.id.saltPrefix:#{null}}")
    private String patientIdSaltPrefix;

    @Value("${idsas.patient.id.saltPostfix:#{null}}")
    private String patientIdSaltPostfix;

    @Value("${soresclient.url}")
    private String soresClientUrl;

    @Value("${soresclient.errorcount.duration:PT10M}")
    private String soresClientErrorCountDuration;

    @Value("${soresclient.error.tolerance:0}")
    private int soresClientErrorTolerance;

    @Value("${idsas.httpclient.pooling.totalconnections:200}")
    private int maxTotalConnections;

    @Value("${idsas.httpclient.pooling.maxconnections.pr.route:20}")
    private int defaultMaxConnectionsPerRoute;

    @Bean
    public LookupService lookupService(BlurringDao blurringDao, AuditLogger auditLogger) {
        // If patientIdSaltEncoded is null or empty
        if (patientIdSaltPrefix == null || patientIdSaltPrefix.isEmpty() || patientIdSaltPostfix == null || patientIdSaltPostfix.isEmpty()) {
            throw new IllegalStateException("patientIdSalt cannot be empty or null");
        }

        return new LookupService(blurringDao, patientIdSaltPrefix, patientIdSaltPostfix, auditLogger, allowedClientSystemNames);
    }

    @Bean
    public OrgBlurringService orgBlurringService(WhitelistDao whitelistDao, OrgBlurringDao orgBlurringDao, SoresClient soresClient) {
        return new OrgBlurringService(whitelistDao, orgBlurringDao, soresClient, allowedClientSystemNames);
    }

    @Bean
    public SoresClient soresClient(PoolingHttpClientFactory poolingHttpClientFactory) {
        HttpClient httpClient = poolingHttpClientFactory.getClient();
        ServiceErrorCollector serviceErrorCollector = new ServiceErrorCollector(Duration.parse(soresClientErrorCountDuration));
        return new SoresClient(httpClient, soresClientUrl, serviceErrorCollector);
    }

    @Bean
    public SoresClientHealthIndicator soresClientHealthIndicator(SoresClient soresClient) {
        return new SoresClientHealthIndicator(soresClient, soresClientErrorTolerance);
    }

    @Bean
    public PoolingHttpClientFactory poolingHttpClientFactory() {
        return new PoolingHttpClientFactory(maxTotalConnections, defaultMaxConnectionsPerRoute);
    }

}
