package dk.nsp.idsas.ws._20230601;

import com.sun.xml.ws.developer.SchemaValidation;
import dk.nsp.idsas.dto.OrganizationIdDto;
import dk.nsp.idsas.dto.PersonIdType;
import dk.nsp.idsas.security.IdsasActorBuilder;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.service.IdsasServiceException;
import dk.nsp.idsas.service.LookupService;
import dk.nsp.idsas.util.FaultCode;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.ws.IdsasAbstractWS;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.*;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.ws.BindingType;
import java.util.List;
import java.util.stream.Collectors;

import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name = "IdsasLookupService", urlPatterns = {"/20230601/lookup"})
@WebService(serviceName = "IdsasLookupService", wsdlLocation = "wsdl/20230601/idsas-lookup.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/",
        portName = "IdsasLookupServicePort")
@BindingType(value = SOAP11HTTP_BINDING)
@SchemaValidation
public class IdsasLookupServiceWS extends IdsasAbstractWS implements IdsasLookupServicePortType {
    private static final Logger logger = Logger.getLogger(IdsasLookupServiceWS.class);

    @Autowired
    private IdsasActorBuilder idsasActorBuilder;

    @Autowired
    private LookupService lookupService;

    @Override
    public BlurredOrganisationsResponseType getBlurredOrganisations(GetBlurredOrganisationsRequestType getBlurredOrganisationsRequest) {
        String patientCpr = getBlurredOrganisationsRequest.getPatientId().getId();
        logger.trace(String.format("Enter get blurred organisations for patient %s", patientCpr));
        PatientIdClassificationType patientIdType = getBlurredOrganisationsRequest.getPatientId().getClassification();

        // Liste af blurrings hentes
        BlurredOrganisationsResponseType entityList = new BlurredOrganisationsResponseType();
        try {
            IdsasActor actor = idsasActorBuilder.buildActor();
            List<OrganizationIdDto> orgs = lookupService.getActiveBlurrings(actor, patientCpr, mapPersonIdType(patientIdType));
            entityList.getOrganisationId().addAll(orgs.stream().map(IdsasLookupServiceWS::mapToOrganisationId).collect(Collectors.toList()));
            return entityList;

        } catch (IdsasServiceException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        } catch (Exception e) {
            logger.error("Failed to get blurred organisations", e);
            throw SoapFaultGenerator.generateSOAPFault(false, "Internal server error",
                    FaultCode.LOOKUP_ERROR.getValue());
        }
    }

    private static OrganisationIdType mapToOrganisationId(OrganizationIdDto dto) {
        OrganisationIdType orgId = new OrganisationIdType();
        orgId.setId(dto.id);
        orgId.setClassification(OrganisationIdClassificationType.fromValue(dto.idType.name()));
        return orgId;
    }

    private static PersonIdType mapPersonIdType(PatientIdClassificationType patientIdType) {
        return PersonIdType.valueOf(patientIdType.value());
    }
}
