package dk.nsp.idsas.ws;

import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;

import static dk.nsp.idsas.service.ServiceSetup.ANNOTATION_CONFIG_CONTEXT;
import static org.mockito.Mockito.*;

public class IdsasContextListenerTest {

    @Mock
    private ServletContextEvent servletContextEvent;

    @Mock
    private ServletContext servletContext;

    private IdsasContextListener listener;

    @Before
    public void setUp() {
        MockitoAnnotations.openMocks(this);
        listener = new IdsasContextListener() {
            @Override
            protected AnnotationConfigApplicationContext getApplicationContext() {
                return mock(AnnotationConfigApplicationContext.class);
            }
        };
        when(servletContextEvent.getServletContext()).thenReturn(servletContext);
    }

    @Test
    public void contextInitialized() {
        // Act
        listener.contextInitialized(servletContextEvent);

        // Assert
        verify(servletContext).setAttribute(eq(ANNOTATION_CONFIG_CONTEXT), any(AnnotationConfigApplicationContext.class));
    }

    @Test
    public void contextDestroyed() {
        // Arrange
        AnnotationConfigApplicationContext applicationContext = mock(AnnotationConfigApplicationContext.class);
        when(servletContext.getAttribute(ANNOTATION_CONFIG_CONTEXT)).thenReturn(applicationContext);

        // Act
        listener.contextDestroyed(servletContextEvent);

        // Assert
        verify(applicationContext).close();
    }
}
