package dk.nsp.idsas.ws._20230601;

import dk.nsp.idsas.dto.OrgIdType;
import dk.nsp.idsas.dto.OrganizationIdDto;
import dk.nsp.idsas.dto.PersonIdType;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.security.IdsasActorBuilder;
import dk.nsp.idsas.service.IdsasServiceException;
import dk.nsp.idsas.service.LookupService;
import dk.nsp.idsas.util.FaultCode;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.*;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import javax.xml.ws.soap.SOAPFaultException;
import java.util.Collections;

import static com.sun.tools.ws.wsdl.parser.Util.fail;
import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.*;

public class IdsasLookupServiceWSTest {

    @Mock
    private IdsasActorBuilder idsasActorBuilder;

    @Mock
    private LookupService lookupService;

    @Mock
    private GetBlurredOrganisationsRequestType getBlurredOrganisationsRequest;

    @Mock
    private PatientIdType patientIdType;

    @InjectMocks
    private IdsasLookupServiceWS idsasLookupServiceWS;

    @Before
    public void setUp() {
        MockitoAnnotations.openMocks(this);
        when(getBlurredOrganisationsRequest.getPatientId()).thenReturn(patientIdType);
        when(patientIdType.getId()).thenReturn("1234567890");
        when(patientIdType.getClassification()).thenReturn(PatientIdClassificationType.CPR);
    }

    @Test
    public void testGetBlurredOrganisations() throws Exception {
        // Arrange
        IdsasActor actor = mock(IdsasActor.class);
        when(idsasActorBuilder.buildActor()).thenReturn(actor);
        OrganizationIdDto orgDto = new OrganizationIdDto("id", OrgIdType.CVR);
        when(lookupService.getActiveBlurrings(actor, "1234567890", PersonIdType.CPR)).thenReturn(Collections.singletonList(orgDto));

        // Act
        idsasLookupServiceWS.getBlurredOrganisations(getBlurredOrganisationsRequest);

        // Assert
        verify(lookupService).getActiveBlurrings(actor, "1234567890", PersonIdType.CPR);
    }

    @Test
    public void testGetBlurredOrganisations_IdsasServiceException() throws Exception {
        // Arrange
        IdsasActor actor = mock(IdsasActor.class);
        when(idsasActorBuilder.buildActor()).thenReturn(actor);
        when(lookupService.getActiveBlurrings(actor, "1234567890", PersonIdType.CPR)).thenThrow(
            new IdsasServiceException("test", FaultCode.LOOKUP_ERROR, true)
        );

        try {
            // Act
            idsasLookupServiceWS.getBlurredOrganisations(getBlurredOrganisationsRequest);
            fail("Expected an IdsasServiceException to be thrown");
        } catch (SOAPFaultException e) {
            // Assert
            assertEquals("test", e.getMessage());
        }

        // Assert
        verify(lookupService).getActiveBlurrings(actor, "1234567890", PersonIdType.CPR);
    }

    @Test
    public void testGetBlurredOrganisations_RuntimeException() throws Exception {
        IdsasActor actor = mock(IdsasActor.class);
        when(idsasActorBuilder.buildActor()).thenReturn(actor);
        when(lookupService.getActiveBlurrings(actor, "1234567890", PersonIdType.CPR)).thenThrow(
                new RuntimeException("test123")
        );

        try {
            idsasLookupServiceWS.getBlurredOrganisations(getBlurredOrganisationsRequest);
            fail("Expected an exception to be thrown");
        } catch (SOAPFaultException e) {
            assertEquals("Internal server error", e.getMessage());
        }

        verify(lookupService).getActiveBlurrings(actor, "1234567890", PersonIdType.CPR);
    }
}