package dk.nsp.idsas.ws._20230601;

import com.sun.xml.ws.developer.SchemaValidation;
import dk.nsp.idsas.security.IdsasActorBuilder;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.service.SaltService;
import dk.nsp.idsas.util.FaultCode;
import dk.nsp.idsas.util.IdsasException;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.ws.IdsasAbstractWS;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.GetCurrentSaltRequest;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.IdsasSaltPortType;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.ws.BindingType;

import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name="IdsasSalt", urlPatterns={"/20230601/salt"})
@WebService(serviceName = "IdsasSalt", wsdlLocation = "wsdl/20230601/idsas-salt.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/",
        portName = "IdsasSaltPort")
@BindingType(value = SOAP11HTTP_BINDING)
@SchemaValidation
public class IdsasSaltWS extends IdsasAbstractWS implements IdsasSaltPortType {
    private static final Logger logger = Logger.getLogger(IdsasSaltWS.class);

    @Autowired
    private IdsasActorBuilder idsasActorBuilder;

    @Autowired
    private SaltService saltService;

    @Override
    public byte[] getCurrentSalt(GetCurrentSaltRequest getCurrentSaltRequest) {
        logger.trace("Enter get current salt");

        try {
            IdsasActor actor = idsasActorBuilder.buildActor();
            return saltService.getCurrentSalt(actor);
        } catch (IdsasException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        } catch (Exception e) {
            logger.error("Failed to get current salt", e);
            throw SoapFaultGenerator.generateSOAPFault(false, "Internal server error",
                    FaultCode.GET_SALT_ERROR.getValue());
        }
    }
}
