package dk.nsp.idsas.ws._20230601;

import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.security.IdsasActorBuilder;
import dk.nsp.idsas.service.IdsasServiceException;
import dk.nsp.idsas.service.SaltService;
import dk.nsp.idsas.util.FaultCode;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.GetCurrentSaltRequest;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.PatientIdClassificationType;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.PatientIdType;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import javax.xml.ws.soap.SOAPFaultException;
import java.nio.charset.StandardCharsets;

import static com.sun.tools.ws.wsdl.parser.Util.fail;
import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.*;

public class IdsasSaltWSTest {

    @Mock
    private IdsasActorBuilder idsasActorBuilder;

    @Mock
    private SaltService saltService;

    @Mock
    private GetCurrentSaltRequest getSaltRequest;

    @Mock
    private PatientIdType patientIdType;

    @InjectMocks
    private IdsasSaltWS idsasSaltWS;

    @Before
    public void setUp() throws IdsasServiceException {
        MockitoAnnotations.openMocks(this);
        when(patientIdType.getId()).thenReturn("1234567890");
        when(patientIdType.getClassification()).thenReturn(PatientIdClassificationType.CPR);
        when(saltService.getCurrentSalt(any())).thenReturn("123".getBytes());
    }

    @Test
    public void getCurrentSalt() throws Exception {
        IdsasActor actor = mock(IdsasActor.class);
        when(idsasActorBuilder.buildActor()).thenReturn(actor);

        byte[] response = idsasSaltWS.getCurrentSalt(getSaltRequest);

        assertEquals("123", new String(response, StandardCharsets.UTF_8));
        verify(saltService).getCurrentSalt(any());
    }

    @Test
    public void getCurrentSalt_IdsasServiceException() throws Exception {
        IdsasActor actor = mock(IdsasActor.class);
        when(idsasActorBuilder.buildActor()).thenReturn(actor);
        IdsasServiceException serviceException = new IdsasServiceException("Test exception", FaultCode.GET_SALT_ERROR, true);
        when(saltService.getCurrentSalt(actor)).thenThrow(serviceException);

        try {
            idsasSaltWS.getCurrentSalt(getSaltRequest);
            fail("Expected an IdsasServiceException to be thrown");
        } catch (SOAPFaultException e) {
            assertEquals("Test exception", e.getMessage());
        }

        verify(saltService).getCurrentSalt(actor);
    }

    @Test
    public void getCurrentSalt_RuntimeException() throws Exception {
        IdsasActor actor = mock(IdsasActor.class);
        when(idsasActorBuilder.buildActor()).thenReturn(actor);
        when(saltService.getCurrentSalt(actor)).thenThrow(new RuntimeException("Test exception"));

        try {
            idsasSaltWS.getCurrentSalt(getSaltRequest);
            fail("Expected an IdsasServiceException to be thrown");
        } catch (SOAPFaultException e) {
            assertEquals("Internal server error", e.getMessage());
        }

        verify(saltService).getCurrentSalt(actor);
    }
}